(ns puppetlabs.activity.http
  (:require
    [clojure.tools.logging :as log]
    [compojure.core :refer [routes context GET POST ANY]]
    [compojure.handler :as handler]
    [cheshire.core :as json]
    [puppetlabs.activity.http.util :as http-util]
    [puppetlabs.activity.http.middleware :as middleware]
    [puppetlabs.activity.postgres :as postgres]
    [slingshot.slingshot :refer [throw+]])
  (:import java.io.StringWriter))

(defn- build-is-current-subject-permitted?
  [is-permitted? req]
  (partial is-permitted? (:shiro-subject req)))

(defn app
  [wrap-authenticated is-permitted? {:keys [db cors-origin-pattern login-path]}]
  (-> (routes
        (context
          "/v1" []

          (GET "/events" {params :params :as req}
               (http-util/response 200 (postgres/get-events db (build-is-current-subject-permitted? is-permitted? req) params)))

          (GET "/events.csv" {params :params :as req}
               (http-util/csv-response 200
                                       (postgres/get-flat-events db (build-is-current-subject-permitted? is-permitted? req) params)
                                       (http-util/csv-filename params))))

        (ANY "*" [:as req]
             (throw+ {:kind :puppetlabs.activity/not-found})))

      wrap-authenticated
      (middleware/wrap-errors-with-explanations login-path)
      handler/api))
