(ns puppetlabs.rbac.services.authz
  "This namespace exposes functions for checking whether or not a
  given Shiro Subject has the given permission strings.

  The primary functions you should be using are
  {is,are}-permitted. The *-session-* functions are for special cases
  where the calling code have nothing but a session id with which to
  identify a subject. They are currently only needed by the Console
  Proxy (which has no access to our authn middleware) and will likely
  be deprecated in 3.5."
  (:require [puppetlabs.rbac.shiro-utils :as shiro]
            [puppetlabs.trapperkeeper.core :refer [defservice]]))

(defprotocol RbacAuthzService
  (is-permitted? [this subject perm-str])
  (are-permitted? [this subject perm-strs])
  (is-session-permitted? [this session-id perm-str])
  (are-session-permitted? [this session-id perm-strs]))


(defservice rbac-authz-service
  RbacAuthzService
  [[:RbacService security-manager]]

  (is-permitted? [this subject perm-str]
                 (shiro/is-permitted? subject perm-str))

  (are-permitted? [this subject perm-strs]
                  (shiro/are-permitted? subject perm-strs))

  (is-session-permitted? [this session-id perm-str]
                         (let [subject (shiro/session-id->subject (security-manager) session-id)]
                           (shiro/is-permitted? subject perm-str)))

  (are-session-permitted? [this session-id perm-strs]
                         (let [subject (shiro/session-id->subject (security-manager) session-id)]
                           (shiro/are-permitted? subject perm-strs))))
